<?php
/* gvHIDRA. Herramienta Integral de Desarrollo Rpido de Aplicaciones de la Generalitat Valenciana
*
* Copyright (C) 2006 Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*  Conselleria d'Infraestructures i Transport
*  Av. Blasco Ibez, 50
*  46010 VALENCIA
*  SPAIN
*  +34 96386 24 83
*  gvhidra@gva.es
*  www.gvhidra.org
*
*/

/**
* Clase Manejadora Estancia
* 
* Creada con Genaro: generador de cdigo de gvHIDRA
* 
* @autor genaro
* @version 1.0
* @license http://opensource.org/licenses/gpl-2.0.php GNU Public License v.2
*/

class Estancia extends gvHidraForm_DB
{
	public function __construct() {

		//Recogemos dsn de conexion
		$conf = ConfigFramework::getConfig();
		$g_dsn = $conf->getDSN('g_dsn');

		$nombreTablas= array('estancia');
		parent::__construct($g_dsn, $nombreTablas);


		
		/************************ QUERYs ************************/
		
		//Consulta del modo de trabajo LIS
		$str_select = "SELECT desde as \"lis_desde\", hasta as \"lis_hasta\", embarcacion_codigo as \"lis_embarcacion_codigo\", puerto_cpuerto as \"puerto_cpuerto\", id as \"lis_id\" FROM estancia";
		$this->setSelectForSearchQuery($str_select);

		//Where del modo de trabajo LIS
		//$str_where = "";
		//$this->setWhereForSearchQuery($str_where);

		//Order del modo de trabajo LIS
		$this->setOrderByForSearchQuery('1');

		//Consulta del modo de trabajo EDI
		//$str_select_listar = "SELECT desde as \"lis_desde\", hasta as \"lis_hasta\", embarcacion_codigo as \"lis_embarcacion_codigo\", puerto_cpuerto as \"lis_puerto_cpuerto\", id as \"lis_id\" FROM estancia";
		$this->setSelectForeditQuery($str_select);
		 
		//Where del modo de trabajo EDI
		//$str_where_listar = "";
		//$this->setWhereForlistQuery($str_where_listar);

		//Order del modo de trabajo EDI
		$this->setOrderByForeditQuery('1');

		/************************ END QUERYs ************************/


		/************************ MATCHINGs ************************/

		//Seccion de matching: asociacion campos TPL y campos BD

		$this->addMatching("lis_desde", "desde", "estancia");
		$this->addMatching("lis_hasta", "hasta", "estancia");
		$this->addMatching("lis_embarcacion_codigo", "embarcacion_codigo", "estancia");
		$this->addMatching("puerto_cpuerto", "puerto_cpuerto", "estancia");
		$this->addMatching("lis_id", "id", "estancia");

		/************************ END MATCHINGs ************************/

		/************************ TYPEs ************************/

		//Fechas: gvHidraDate type

		//Strings: gvHidraString type		
		$string = new gvHidraString(false, 5);
		$this->addFieldType('lis_puerto_cpuerto', $string);
		
			
		//Integers: gvHidraInteger type
		$int = new gvHidraInteger(false, 4);
		$this->addFieldType('lis_id', $int);
		

		//Floats: gvHidraFloat type

		/************************ END TYPEs ************************/
		
		/************************ COMPONENTS ************************/
		
		//Declaracion de Listas y WindowSelection
		$listaEmbarcaciones = new gvHidraList('lis_embarcacion_codigo','EMBARCACIONES');
		$this->addList($listaEmbarcaciones);
		//La definicin debe estar en el AppMainWindow.php
		
		
		/************************ END COMPONENTS ************************/						

		//Asociamos con la clase maestro
		$this->addPadre("Puerto");
	}//End construct

	/************************ CRUD METHODs ************************/

	/**
	* metodo preRecargar
	* 
	* @access public
	* @param object $objDatos
	* 
	* Incorpore aqui la logica para parametrizar la carga del detalle. Por ejemplo:
	* - Incluir condiciones.
	* - Cancelar la accion. 
	*/	
	public function preBuscar($objDatos) {
		
		return 0;
	}

	/**
	* metodo postRecargar
	* 
	* @access public
	* @param object $objDatos
	* 
	* Incorpore aqui la logica para parametrizar los datos obtenidos. Por ejemplo:
	* - Completar la informacion obtenida.
	* - Cambiar el color de las filas dependiendo de su valor
	*/	
	public function postRecargar($objDatos) {
		
		return 0;
	}

	/**
	* metodo preInsertar
	* 
	* @access public
	* @param object $objDatos
	* 
	* Incorpore aqui la logica para parametrizar los datos a insertar. Por ejemplo:
	* - Calcular el valor de una secuencia.
	* - Cancelar la accin de insercion.
	*/		
	public function preInsertar($objDatos) {
		$mDatos = $objDatos->getAllTuplas();
		foreach($mDatos as $indice=>$tupla)
		{
			$puerto = $tupla['lis_puerto_cpuerto'];
			$embarcacion = $tupla['lis_embarcacion_codigo'];
			$desde = $tupla['lis_desde'];
			$hasta = $tupla['lis_hasta'];
			// Preparamos la fecha para adaptarla al tipo de la BD
			$desde = $this->getConnection()->prepararFecha($desde);
			$hasta = $this->getConnection()->prepararFecha($hasta);
			$ocupado = $this->existeEstancia($puerto,$embarcacion,$desde,$hasta);
			if ($ocupado == 0)
			{
				$this->showMessage('APL-03',array($embarcacion));
				return -1;
			}
		}
		return 0;
	}
	
	/**
	* metodo postInsertar
	* 
	* @access public
	* @param object $objDatos
	* 
	* Incorpore aqui la logica para completar la operacion de insercion. Por ejemplo:
	* - Insertar en una segunda tabla.
	*/		
	public function postInsertar($objDatos) {
		
		return 0;
	}

	/**
	* metodo preModificar
	* 
	* @access public
	* @param object $objDatos
	* 
	* Incorpore aqui la logica para parametrizar la operacion de actualizacion. Por ejemplo:
	* - Calcular valores derivados.
	* - Cancelar la accin de actualizacion.
	*/
	public function preModificar($objDatos) {
		$mDatos = $objDatos->getAllTuplas();
		foreach($mDatos as $indice=>$tupla)
		{
			$puerto = $tupla['lis_puerto_cpuerto'];
			$embarcacion = $tupla['lis_embarcacion_codigo'];
			$desde = $tupla['lis_desde'];
			$hasta = $tupla['lis_hasta'];
			// Preparamos la fecha para adaptarla al tipo de la BD
			$desde = $this->getConnection()->prepararFecha($desde);
			$hasta = $this->getConnection()->prepararFecha($hasta);
			$ocupado = $this->existeEstancia($puerto,$embarcacion,$desde,$hasta);
			if ($ocupado == 0)
			{
				$this->showMessage('APL-04',array($embarcacion));
				return -1;
			}
		}
		return 0;
	}
	
	/**
	* metodo postModificar
	* 
	* @access public
	* @param object $objDatos
	* 
	* Incorpore aqui la logica para completar la operacion de actulizacion. Por ejemplo:
	* - Actualizar en una segunda tabla
	*/	
	public function postModificar($objDatos) {
		
		return 0;
	}
	
	/**
	* metodo preBorrar
	* 
	* @access public
	* @param object $objDatos
	* 
	* Incorpore aqui la logica para parametrizar la operacion de borrado. Por ejemplo:
	* - Cancelar la accin de borrado.
	*/	
	public function preBorrar($objDatos) {
		
		return 0;
	}
	
	/**
	* metodo postBorrar
	* 
	* @access public
	* @param object $objDatos
	* 
	* Incorpore aqui la logica para completar la operacion de borrado. Por ejemplo:
	* - Borrar en una segunda tabla
	*/	
	public function postBorrar($objDatos) {
		
		return 0;
	}
	
	/**
	* metodo preNuevo
	* 
	* @access public
	* @param object $objDatos
	* 
	* Incorpore aqui los valores por defecto antes de insertar.
	*/	
	public function preNuevo($objDatos) {
		
		return 0;
	}
	
	/**
	* metodo preIniciarVentana
	* 
	* @access public
	* @param object $objDatos
	* 
	* Incorpore aqui la logica a ejecutar cuando entra en la ventana. Por ejemplo:
	* - Puede comprobar que el usuario tiene los permisos necesarios.
	*/	
	public function preIniciarVentana($objDatos) {
		
		return 0;
	}
	
	/************************ END CRUD METHODs ************************/
	
	public function existeEstancia($puerto, $embarcacion, $desde, $hasta) 
	{
		$consulta = "SELECT count(*) as \"ocupado\" FROM estancias WHERE embarcacion_codigo = '".$embarcacion."' and puerto_cpuerto <> '".$puerto."' AND (desde BETWEEN '".$desde."' AND '".$hasta."') OR (hasta BETWEEN '".$desde."' AND '".$hasta."')";
		$res = $this->consultar($consulta);
		if ($res[0]['ocupado']>0)
			return 0;
		else
			return -1;
	}
	
	/**
	* metodo accionesParticulares
	* 
	* @access public
	* @param string $str_accion
	* @param object $objDatos
	* 
	* Incorpore aqui la logica de sus acciones particulares. 
	* -En el parametro $str_accion aparece el id de la accion.
	* -En el parametro $objDatos esta la informacion de la peticion. Recuerde que debe fijar la operacion
	* con el metodo setOperacion.
	*/	
	public function accionesParticulares($str_accion, $objDatos) {
        
		throw new Exception('Se ha intentado ejecutar la accin '.$str_accion.' y no est programada.');        
    }
	
}//End 

?>